/*
  Copyright 2021 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner.friend

import com.kakao.sdk.auth.network.kapiWithOAuth
import com.kakao.sdk.network.ApiCallback
import com.kakao.sdk.network.ApiFactory
import com.kakao.sdk.partner.friend.model.PickerFriendFilter
import com.kakao.sdk.partner.friend.model.PickerFriendType
import com.kakao.sdk.partner.friend.network.FriendApi
import com.kakao.sdk.partner.friend.network.model.Me
import com.kakao.sdk.partner.friend.network.model.PickerFriends
import com.kakao.sdk.partner.talk.model.FriendFilter
import com.kakao.sdk.partner.talk.model.FriendType
import com.kakao.sdk.partner.talk.model.PartnerFriend
import com.kakao.sdk.partner.talk.model.PartnerFriendsContext

/**
 * @suppress
 */
internal object Repository {
    private val friendApi: FriendApi = ApiFactory.kapiWithOAuth.create(FriendApi::class.java)
    private var friendsContext: PartnerFriendsContext? = null
    var friends = mutableListOf<PartnerFriend>()
    var me: Me? = null

    fun loadFriendsData(
        friendType: PickerFriendType,
        friendFilter: PickerFriendFilter,
        countryCodes: List<String>?,
        callback: (error: Throwable?) -> Unit
    ) {
        if (friendsContext == null) {
            var filter = FriendFilter.NONE
            if (friendFilter == PickerFriendFilter.INVITABLE) {
                filter = FriendFilter.INVITABLE
            } else if (friendFilter == PickerFriendFilter.REGISTERED) {
                filter = FriendFilter.REGISTERED
            }
            var type = FriendType.KAKAO_TALK
            if (friendType == PickerFriendType.KAKAO_STORY) {
                type = FriendType.KAKAO_STORY
            } else if (friendType == PickerFriendType.KAKAO_TALK_AND_STORY) {
                type = FriendType.KAKAO_TALK_AND_STORY
            }

            friendsContext = PartnerFriendsContext(
                friendFilter = filter,
                friendType = type,
                countryCodes = countryCodes,
                limit = Constants.PARTNER_FRIENDS_API_LIMIT
            )
        }

        friendsContext?.let { context ->
            pickerFriendsForPartner(context) { data, error ->
                if (error != null) {
                    callback(error)
                } else if (data != null) {
                    friends.addAll(data.friends.elements)
                    if (data.friends.afterUrl == null) {
                        callback(null)
                        me = data.me
                    } else {
                        friendsContext = PartnerFriendsContext(data.friends.afterUrl!!)
                        loadFriendsData(friendType, friendFilter, countryCodes, callback)
                    }
                }
            }
        }
    }

    private fun pickerFriendsForPartner(
        context: PartnerFriendsContext,
        callback: (friends: PickerFriends<PartnerFriend>?, error: Throwable?) -> Unit
    ) {
        friendApi.friends(
            context.friendType,
            context.friendFilter,
            context.friendOrder,
            context.offset,
            context.limit,
            context.order,
            context.countryCodes?.joinToString(",")
        ).enqueue(object : ApiCallback<PickerFriends<PartnerFriend>>() {
            override fun onComplete(model: PickerFriends<PartnerFriend>?, error: Throwable?) {
                callback(model, error)
            }
        })
    }

    fun clearData() {
        me = null
        friends.clear()
        friendsContext = null
    }
}