/*
  Copyright 2021 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner.friend.adapter

import android.view.LayoutInflater
import android.view.ViewGroup
import androidx.constraintlayout.widget.ConstraintLayout
import androidx.constraintlayout.widget.ConstraintLayout.LayoutParams.*
import androidx.constraintlayout.widget.ConstraintSet
import androidx.recyclerview.widget.RecyclerView
import com.kakao.sdk.partner.friend.imageloader.ImageLoader
import com.kakao.sdk.partner.friend.model.Item
import com.kakao.sdk.partner.friend.util.dpToPixel
import com.kakao.sdk.partner.friend.util.getDefaultProfileImage
import com.kakao.sdk.v2.partner.friend.databinding.KakaoSdkItemPickedFriendBinding

/**
 * @suppress
 */
internal class PickedFriendsAdapter(
    private val isPopup: Boolean,
    private val pickedFriends: MutableList<Item.User>,
    private val removeFriendCallback: (Item.User) -> Unit
) : RecyclerView.Adapter<PickedFriendsAdapter.PickedFriendViewHolder>() {

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): PickedFriendViewHolder {
        val inflater = LayoutInflater.from(parent.context)
        return PickedFriendViewHolder(
            isPopup,
            KakaoSdkItemPickedFriendBinding.inflate(inflater, parent, false)
        ) {
            removePickedFriend(it)
            removeFriendCallback(it)
        }
    }

    override fun onBindViewHolder(holder: PickedFriendViewHolder, position: Int) {
        holder.bind(pickedFriends[position])
    }

    override fun getItemCount(): Int = pickedFriends.size

    fun addPickedFriend(friend: Item.User) {
        val isFirst = pickedFriends.isEmpty()
        pickedFriends.add(0, friend)

        // 첫 번째 친구 추가할 때 깜빡거리는 현상 제거
        if (isFirst) {
            notifyDataSetChanged()
        } else {
            notifyItemInserted(0)
        }
    }

    fun removePickedFriend(Friend: Item.User) {
        val index = pickedFriends.indexOf(Friend)
        pickedFriends.removeAt(index)
        notifyItemRemoved(index)
    }

    class PickedFriendViewHolder(
        private val isPopup: Boolean,
        private val binding: KakaoSdkItemPickedFriendBinding,
        private val removeFriendCallback: (Item.User) -> Unit
    ) :
        RecyclerView.ViewHolder(binding.root) {

        fun bind(user: Item.User) {
            binding.apply {
                if (isPopup) {
                    setPopupLayout()
                }
                
                val defaultProfileImage =
                    getDefaultProfileImage(user.uuid.hashCode(), root.context).resourceId
                ImageLoader.get(itemView.context).load(user.profileImageUrl)
                    .placeHolder(defaultProfileImage).into(profileIv)
                pickedNicknameTv.text = user.nickname
                root.setOnClickListener {
                    removeFriendCallback(user)
                }
            }
        }

        private fun setPopupLayout() {
            binding.apply {
                root.apply {
                    layoutParams = ConstraintLayout.LayoutParams(WRAP_CONTENT, WRAP_CONTENT).apply {
                        val marginHorizontal = dpToPixel(1f, resources)
                        setMargins(marginHorizontal, 0, marginHorizontal, 0)
                    }
                }
                profileIv.apply {
                    ConstraintSet().apply {
                        clone(root)
                        connect(id, TOP, PARENT_ID, TOP, dpToPixel(6.5f, resources))
                    }.applyTo(root)
                }
                pickedNicknameTv.apply {
                    ConstraintSet().apply {
                        clone(root)
                        connect(id, TOP, profileIv.id, BOTTOM, dpToPixel(5.5f, resources))
                        connect(id, BOTTOM, PARENT_ID, BOTTOM, dpToPixel(6f, resources))
                    }.applyTo(root)
                }
            }
        }
    }
}